import os
import sys
from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
from fastapi.responses import FileResponse
from .routers import front_route, proxy_route, proxy_need_auth_route
from .middlewares.exception_handler import ExceptionHandlingMiddleware
from zylo_docs.services.openapi_service import OpenApiService 
from zylo_docs.logging import NoZyloDocsLogFilter
from dotenv import load_dotenv

# config.py 파일의 위치를 기준으로 상위 폴더의 .env 파일을 찾습니다.
dotenv_path = os.path.join(os.path.dirname(__file__), "..", ".env")
load_dotenv(dotenv_path=dotenv_path)
# 로깅 삭제하는 코드 개발 모드에서는 주석 처리
if os.getenv("DEV") != "dev":
    NoZyloDocsLogFilter().setup_logging()

def _get_server_info():
    host = "localhost"
    port = "8000"

    try:
        if "--host" in sys.argv:
            host = sys.argv[sys.argv.index("--host") + 1]
        if "--port" in sys.argv:
            port = sys.argv[sys.argv.index("--port") + 1]
    except (ValueError, IndexError):
        pass  # Ignore if parsing fails, stick to defaults
    
    return host, str(port)

def set_initial_openapi_spec(app: FastAPI):
    openapi_json = app.openapi()
    app.state.openapi_service.set_current_spec(openapi_json)
    
    host, port = _get_server_info()

    message = f"""
┌───────────────────────────────────────────────────────────────────────────┐
│                                                                           │
│  🚀 zylo-docs is running locally!                                         │
│                                                                           │
│  🔗 http://{host}:{port}/zylo-docs                                       │
│                                                                           │
│  Check your API spec using the zylo-docs web app.                         │
│                                                                           │
└───────────────────────────────────────────────────────────────────────────┘
"""
    print(message)

def zylo_docs(app: FastAPI):
    @app.on_event("startup")
    async def on_startup():
        set_initial_openapi_spec(app)
    if not hasattr(app.state, 'openapi_service'):
        app.state.openapi_service = OpenApiService()
        
    app.mount("/static", StaticFiles(directory=os.path.join(os.path.dirname(__file__), "static")), name="static")

    app.include_router(front_route.router, prefix="/zylo-docs", tags=["front"])
    app.include_router(proxy_need_auth_route.router, prefix="/zylo-docs/api", tags=["proxy_auth"])
    app.include_router(proxy_route.router, prefix="/zylo-docs/api", tags=["proxy"])
    app.add_middleware(ExceptionHandlingMiddleware)

    @app.get("/zylo-docs{full_path:path}", include_in_schema=False)
    async def serve_react_app():
        return FileResponse(os.path.join(os.path.dirname(__file__), "static", "index.html"))



from pydantic import BaseModel, Field
from typing import Any, Dict, Optional

class SchemaResponseModel(BaseModel):
    success: bool
    message: str
    data: Any
class APIResponseBodyModel(BaseModel):
    value: Optional[Dict[str, Any]] = Field(default_factory=dict)

class APIInputModel(BaseModel):
    path_params: Optional[Dict[str, Any]] = Field(default_factory=dict)
    query_params: Optional[Dict[str, Any]] = Field(default_factory=dict)
    cookie_params: Optional[Dict[str, Any]] = Field(default_factory=dict)
    headers: Optional[Dict[str, Any]] = Field(default_factory=dict)
    body: APIResponseBodyModel = Field(default_factory=APIResponseBodyModel)
class APIRequestModel(BaseModel):
    method: str
    path: str
    input: APIInputModel = Field(default_factory=APIInputModel)




import httpx
from fastapi import HTTPException
from zylo_docs.config import EXTERNAL_API_BASE
async def get_spec_content_by_id(spec_id: str, client: httpx.AsyncClient, access_token: str, source: str = "tuned") -> dict:
    try:
        resp = await client.get(f"{EXTERNAL_API_BASE}/specs/{spec_id}?source={source}", headers={"Authorization": f"Bearer {access_token}"})
        resp.raise_for_status()
        response_data = resp.json()
        spec_content = response_data.get("data", {}).get("spec_content")
        if not spec_content:
            raise HTTPException(status_code=404, detail="Spec content not found in the external API response.")
        return spec_content
    except httpx.HTTPStatusError as exc:
        raise HTTPException(
            status_code=exc.response.status_code,
            detail={
                "message": f"specs/{spec_id} endpoint returned an error",
                "response": exc.response.json(),
            }
        )





from fastapi import Depends, HTTPException

from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

security = HTTPBearer(auto_error=False)
async def verify_token(credentials: HTTPAuthorizationCredentials = Depends(security)):

    if credentials is None:
        # 토큰이 아예 없는 경우, 원하는 에러를 여기서 발생시킵니다.
        raise HTTPException(
            status_code=401,
            detail={
                "success": False,
                "message": "user's authorization is required, please sign in",
                "data": {
                    "code": "SIGN_IN_REQUIRED",
                    "details": "user's authorization is required."
                }
        }
    )
    access_token = credentials.credentials
    if not access_token:
        raise HTTPException(
                status_code=401,
                detail={
                    "success": False,
                    "message": "Invalid or missing access token",
                    "data": {
                        "code": "INVALID_ACCESS_TOKEN",
                        "details": "No operation found with operationId 'invalidId'"
                    }
                }
            )
    return credentials



from collections import defaultdict
import copy
from zylo_docs.services.openapi_service import OpenApiService

def parse_openapi_paths(paths):
    grouped = defaultdict(list)

    for path, methods in paths.items():
        for method, info in methods.items():
            tag = (info.get("tags") or ["default"])[0]

            grouped[tag].append({
                "operationId": info.get("operationId", ""),
                "method": method.upper(),
                "path": path,
                "summary": info.get("summary", "")
            })
    return {
        "operationGroups": [
            {
                "tag": tag,
                "operations": operation
            } for tag, operation in grouped.items()
        ]
    
    }

async def get_user_operation(request):
    service: OpenApiService = request.app.state.openapi_service
    openapi_json = service.get_current_spec()
    result = parse_openapi_paths(openapi_json.get("paths", {}))
    return result

def resolve_ref(obj, components):
    if isinstance(obj, dict):
        if "$ref" in obj:
            ref_path = obj['$ref'].strip('#/').split('/')
            ref = components
            for key in ref_path[1:]:
                if not isinstance(ref, dict) or key not in ref:
                    return obj
                ref = ref[key]
            return resolve_ref(ref, components)
        else:
            return {k: resolve_ref(v, components) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [resolve_ref(item, components) for item in obj]
    else:
        return obj
def resolve_testcase_inputs(info):
    testcase = {}
    parameters = info.get("parameters", [])
    for param in parameters:
        if param.get("in")=="query":
            if param.get("examples"):
                for key, value in param["examples"].items():
                    if key not in testcase:
                        testcase[key] = {}
                    if testcase[key].get("query_params") is None:
                        testcase[key]["query_params"] = {}
                    testcase[key]["query_params"][param["name"]] = value.get("value")
        if param.get("in")=="path":
            if param.get("examples"):
                for key, value in param["examples"].items():
                    if key not in testcase:
                        testcase[key] = {}
                    if testcase[key].get("path_params") is None:
                        testcase[key]["path_params"] = {}
                    testcase[key]["path_params"][param["name"]] = value.get("value")
    requestBody_examples = info.get("requestBody", {}).get("content",{}).get("application/json", {}).get("examples", {})
    for key, value in requestBody_examples.items():
        if key not in testcase:
            testcase[key] = {}
        if testcase[key].get("body") is None:
            testcase[key]["body"] = {}
        testcase[key]["body"] = value
    return testcase

def parse_openapi_paths_by_method(paths, components, path, target_method):
    path_item = paths.get(path, {})
    if not path_item:
        return None

    resolved_path_item = copy.deepcopy(path_item)
    for method, info in resolved_path_item.items():
        if method != target_method:
            continue
        if info.get("requestBody"):
            info["requestBody"] = resolve_ref(info["requestBody"], components)
        if info.get("responses"):
            info["responses"] = resolve_ref(info["responses"], components)
        testcase = resolve_testcase_inputs(info)
        info["testcase_inputs"] = testcase
        
    return resolved_path_item

async def get_user_operation_by_path(request, path, method):
    service: OpenApiService = request.app.state.openapi_service
    openapi_json = service.get_current_spec()
    components = openapi_json.get("components", {})

    result = parse_openapi_paths_by_method(openapi_json.get("paths", {}), components, path, method)
    return result



async def get_cur_test_case(request, path, method):
    service: OpenApiService = request.app.state.openapi_service
    openapi_json = service.get_current_spec()
    result = copy.deepcopy(openapi_json.get("paths", {}).get(path, {}).get(method))
    if result:
        result.pop("responses", None)
    
    return result 
async def update_current_spec(request, new_spec, path, method):
    service: OpenApiService = request.app.state.openapi_service
    openapi_json = service.get_current_spec()
    for param in openapi_json["paths"][path][method].get("parameters", []):
        if param.get("in") == "path":
            if param.get("examples"):
                for new_param in new_spec["parameters"]:
                    if new_param.get("in") == "path" and new_param.get("examples"):
                        param["examples"] = new_param["examples"]
        if param.get("in") == "query":
            if param.get("examples"):
                for new_param in new_spec["parameters"]:
                    if new_param.get("in") == "query" and new_param.get("examples"):
                        param["examples"] = new_param["examples"]


    if "requestBody" in openapi_json["paths"][path][method]:
        if openapi_json["paths"][path][method]["requestBody"].get("content", {}).get("application/json", {}).get("examples"):
            new_body_example = new_spec["requestBody"].get("content", {}).get("application/json", {}).get("examples", {})
            openapi_json["paths"][path][method]["requestBody"]["content"]["application/json"]["examples"] = new_body_example
    service.set_current_spec(openapi_json)
    return 



from fastapi import APIRouter, Request, Response, Depends, Query
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from typing import Optional
import json
import httpx
from io import BytesIO
from pydantic import BaseModel, Field
from enum import Enum
from fastapi.responses import JSONResponse
from zylo_docs.services.openapi_service import OpenApiService
from zylo_docs.services.hub_server_service import get_spec_content_by_id
from zylo_docs.services.user_server_service import get_cur_test_case, update_current_spec
from zylo_docs.security import verify_token
from zylo_docs.config import EXTERNAL_API_BASE
from pydantic import BaseModel
import logging
logger = logging.getLogger(__name__)


router = APIRouter()
class DocTypeEnum(str, Enum):
    internal = "internal"
    public = "public"
    partner = "partner"

class ZyloAIRequestBody(BaseModel):
    title: str = Field(..., description="Title of the OpenAPI spec")
    version: str = Field(..., description="Version of the spec")
    doc_type: DocTypeEnum
    
class ZyloAIUserContextRequestBody(BaseModel):
    title: str = Field(..., description="Title of the OpenAPI spec")
    version: str = Field(..., description="Version of the spec")
    doc_type: DocTypeEnum
    spec_id: str = Field(..., description="Spec ID for current spec")
    user_context: Optional[str] = Field(None, description="User context for the spec")
    
class InviteRequestBody(BaseModel):
    emails: list[str] = Field(..., description="List of emails to invite")
class TestCasePatchBody(BaseModel):
    spec_id: str = Field(..., description="Spec ID for the test case")
    path: str = Field(..., description="Operation ID for the test case")
    method: str = Field(..., description="Test case method")



@router.post("/zylo-ai", include_in_schema=False)
async def create_zylo_ai(request: Request, body: ZyloAIRequestBody, credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    service: OpenApiService = request.app.state.openapi_service
    openapi_dict = service.get_current_spec()

    openapi_json_content = json.dumps(openapi_dict, indent=2).encode('utf-8')
    openapi_file_like = BytesIO(openapi_json_content)
    # timeout 버그를 피하기 위해 timeout을 None으로 설정
    timeout = httpx.Timeout(timeout=None, connect=None, read=None, write=None)
    async with httpx.AsyncClient(timeout=timeout) as client:
        files_for_upload = {
            'file': ('openapi.json', openapi_file_like, 'application/json')
        }
        text_data = {
            "title": body.title,
            "version": body.version,
            "doc_type": body.doc_type.value,
        }
        try:
            resp = await client.post(
                f"{EXTERNAL_API_BASE}/zylo-ai", 
                files=files_for_upload, 
                data=text_data,
                headers={
                    "Authorization": f"Bearer {access_token}"
                }
            )
            resp.raise_for_status()
        
        except httpx.HTTPStatusError as exc:
            return Response(
                content=exc.response.content,
                status_code=exc.response.status_code,
                media_type=exc.response.headers.get("content-type")
            )
        response_json = resp.json()
        spec_id = response_json.get("data", {}).get("id")
        if not spec_id:
            return Response(content="Response JSON does not contain 'data.id' field.",status_code=400)
        try:
            tuned_spec_content = await get_spec_content_by_id(spec_id, client, access_token)
            service.set_current_spec(tuned_spec_content)
            return JSONResponse(
                content={
                    "success": True,
                    "message": f"Successfully tuned and applied spec_id: {spec_id}",
                }
            )
        
        except httpx.HTTPStatusError as exc:
            return JSONResponse(
                status_code=exc.response.status_code,
                content={
                    "success": False,
                    "message": "Failed to retrieve tuned spec content",
                    "details": f"specs/{spec_id} endpoint returned an error",
                }
            )

@router.post("/zylo-ai/user-context", include_in_schema=False)
async def create_zylo_ai_user_context(request: Request, body: ZyloAIUserContextRequestBody, credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    service: OpenApiService = request.app.state.openapi_service
    openapi_dict = service.get_current_spec()
    openapi_json_content = json.dumps(openapi_dict, indent=2).encode('utf-8')
    openapi_file_like = BytesIO(openapi_json_content)
    timeout = httpx.Timeout(timeout=None, connect=None, read=None, write=None)
    async with httpx.AsyncClient(timeout=timeout) as client:
        files_for_upload = {
            'file': ('openapi.json', openapi_file_like, 'application/json')
        }
        text_data = {
            "title": body.title,
            "version": body.version,
            "doc_type": body.doc_type.value,
            "base_spec_id": body.spec_id,
            "user_context": body.user_context
        }
        try:
            resp = await client.post(
                f"{EXTERNAL_API_BASE}/zylo-ai/user-context", 
                files=files_for_upload, 
                data=text_data,
                headers={
                    "Authorization": f"Bearer {access_token}"
                }
            )
            resp.raise_for_status()
        
        except httpx.HTTPStatusError as exc:
            return Response(
                content=exc.response.content,
                status_code=exc.response.status_code,
                media_type=exc.response.headers.get("content-type")
            )
        response_json = resp.json()
        spec_id = response_json.get("data", {}).get("id")
        if not spec_id:
            return Response(content="Response JSON does not contain 'data.id' field.",status_code=400)
        try:
            tuned_spec_content = await get_spec_content_by_id(spec_id, client, access_token)
            service.set_current_spec(tuned_spec_content)
            return JSONResponse(
                content={
                    "success": True,
                    "message": f"Successfully tuned and applied spec_id: {spec_id}",
                }
            )
        
        except httpx.HTTPStatusError as exc:
            return JSONResponse(
                status_code=exc.response.status_code,
                content={
                    "success": False,
                    "message": "Failed to retrieve tuned spec content",
                    "details": f"specs/{spec_id} endpoint returned an error",
                }
            )
        
@router.get("/specs/me",include_in_schema=False)
async def get_spec(credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    async with httpx.AsyncClient() as client:
        try:
            resp = await client.get(f"{EXTERNAL_API_BASE}/specs/me", headers={"Authorization": f"Bearer {access_token}"})
            resp.raise_for_status()
            return resp.json()
        except httpx.HTTPStatusError as exc:
            return Response(
                content=exc.response.content,
                status_code=exc.response.status_code,
                media_type=exc.response.headers.get("content-type")
            )
        
        
    return Response(
        content=resp.content,
        media_type=resp.headers.get("content-type")
    )

@router.post("/toggle/spec/{spec_id}", include_in_schema=False)
async def get_specs_me_by_id(request: Request, spec_id: str, source: str = Query(...), credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    async with httpx.AsyncClient() as client:
        try:
            spec_content = await get_spec_content_by_id(spec_id, client, access_token, source=source)
            service: OpenApiService = request.app.state.openapi_service
            service.set_current_spec(spec_content)
            return JSONResponse(
                    content={
                        "success": True,
                        "message": "Spec retrieved successfully",
                    }
                )
        except httpx.HTTPStatusError as exc:
            return Response(
                content=exc.response.content,
                status_code=exc.response.status_code,
                media_type=exc.response.headers.get("content-type")
            )

@router.patch("/testcases", include_in_schema=False)
async def create_test_case(request: Request, body: TestCasePatchBody, credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    spec_id, path, method = body.spec_id, body.path, body.method.lower()
    cur_test_case = await get_cur_test_case(request, path, method)
    request_data = {"spec_data": cur_test_case, "spec_id": spec_id} 
    timeout = httpx.Timeout(timeout=None, connect=None, read=None, write=None)
    async with httpx.AsyncClient(timeout=timeout) as client:
        try:
            resp = await client.post(f"{EXTERNAL_API_BASE}/zylo-ai/testcases", json=request_data, headers={"Authorization": f"Bearer {access_token}"})
            resp.raise_for_status()
            await update_current_spec(request, resp.json().get("data", {}).get("tuned_json",{}), path, method)
            return resp.json()
        except httpx.HTTPStatusError as exc:
            return Response(
                content=exc.response.content,
                status_code=exc.response.status_code,
                media_type=exc.response.headers.get("content-type")
            )


@router.post("/pivot-current-spec/{spec_id}", include_in_schema=False)
async def get_spec_by_id(request: Request, spec_id: str, credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    if spec_id == "original":
        service: OpenApiService = request.app.state.openapi_service
        service.set_current_spec(request.app.openapi())
        return JSONResponse(
            content={
                "success": True,
                "message": "Original OpenAPI spec retrieved successfully",
            }
        )
    else:
        async with httpx.AsyncClient() as client:
            try:
                spec_content = await get_spec_content_by_id(spec_id, client, access_token)
                service: OpenApiService = request.app.state.openapi_service
                service.set_current_spec(spec_content)
                return JSONResponse(
                    content={
                        "success": True,
                        "message": "Spec retrieved successfully",
                    }
                )
            except httpx.HTTPStatusError as exc:
                return JSONResponse(
                    status_code=exc.response.status_code,
                    content={
                        "success": False,
                        "message": "Failed to retrieve spec content",
                        "details": f"specs/{spec_id} endpoint returned an error",
                    }
                )
            

    
@router.post("/projects/{dummy_id}/specs/{spec_id}/invite", include_in_schema=False)
async def get_project_members(dummy_id: str, spec_id: str, body: InviteRequestBody , credentials: HTTPAuthorizationCredentials = Depends(verify_token)):
    access_token = credentials.credentials
    project_id = ""
    emails = body.emails
    timeout = httpx.Timeout(timeout=None, connect=None, read=None, write=None)
    async with httpx.AsyncClient(timeout=timeout) as client:
        try:
            resp = await client.get(f"{EXTERNAL_API_BASE}/projects", headers={"Authorization": f"Bearer {access_token}"})
            resp.raise_for_status()
            project_list = resp.json().get("data", [])
            if not project_list:
                return JSONResponse(
                    status_code=400,
                    content={
                        "success": False,
                        "message": "Since the user has never run zylo-ai, the project hasn’t been created yet.",
                        "error": {
                            "code": "PROJECT_NOT_FOUND",
                            "details": "Please run zylo-ai first to create a project and then try again."
                        }
                    }
                )
            # 지금은 아이디 하나당 하나의 프로젝트만 있다고 가정하기 때문에 인덱스0에서 Project_id를 가져옴
            project_id = project_list[0].get("project_id")
        
        except httpx.HTTPStatusError as exc:
            logger.warning(f"HTTP error during project fetch: {exc}")
            return JSONResponse(
                status_code=exc.response.status_code,
                content={
                    "success": False,
                    "message": "Failed to fetch project list.",
                    "error": {
                        "code": "EXTERNAL_API_ERROR",
                        "details": exc.response.text
                    }
                }
            )

    async with httpx.AsyncClient(timeout=timeout) as client:
        resp = await client.post(
            f"{EXTERNAL_API_BASE}/projects/{project_id}/specs/{spec_id}/invite",
            headers={"Authorization": f"Bearer {access_token}"},
            json={"emails": emails}
        )
        resp.raise_for_status()
        return JSONResponse(status_code=200, content=resp.json())





from fastapi import APIRouter,Query, Request,HTTPException
from zylo_docs.services.user_server_service import get_user_operation,get_user_operation_by_path
from zylo_docs.schemas.schema_data import SchemaResponseModel
from zylo_docs.schemas.schema_data import APIRequestModel
from zylo_docs.services.openapi_service import OpenApiService
from fastapi.responses import JSONResponse
from zylo_docs.utils.error_response import make_error_response
import urllib.parse
import httpx

router = APIRouter()
@router.get("/openapi.json", include_in_schema=False)
async def get_openapi_json(request: Request):
    openapi = request.app.openapi()
    return{
        "success": True,
        "message": "OpenAPI JSON retrieved successfully",
        "data":openapi
    }
# @router.get("/operation", response_model=SchemaResponseModel, include_in_schema=False)
# async def get_operation(request: Request):
#     try:
#         result = await get_user_operation(request)
#         if not result["operationGroups"]:
#             raise HTTPException(
#                 status_code=404,
#                 detail={
#                     "success": False,
#                     "message": "Operation not found",
#                     "data": {
#                         "code": "OPERATION_NOT_FOUND",
#                         "details": "No operation found with operationId 'invalidId'"
#                     }
#                 }
#             )

#         return {
#             "success": True,
#             "message": "All operation listed",
#             "data": result
#         }
#     except Exception as e:
#         raise ValueError(f"Unexpected error: {e}")

@router.get("/operation/by-path", include_in_schema=False)
async def get_operation_by_path(
    request: Request,
    path: str = Query(..., description="조회할 operationId"),
    method: str = Query(..., description="HTTP 메소드")
):
    result = await get_user_operation_by_path(request, path, method)
    if not result or not result.get(method):
        raise HTTPException(
            status_code=404,
            detail={
                "success": False,
                "message": "Operation not found",
                "data": {
                    "code": "OPERATION_NOT_FOUND",
                    "details": f"No operation found with operationId '{path}'"
                }
            }
        )
    return {
        "success": True,
        "message": "Operation retrieved successfully",
        "data": result.get(method)
    }
@router.get("/current-spec", include_in_schema=False)
async def get_current_spec(request: Request):
    service: OpenApiService = request.app.state.openapi_service
    openapi_json = service.get_current_spec()
    return {
        "success": True,
        "message": "Current OpenAPI spec retrieved successfully",
        "data": openapi_json
    }


@router.post("/test-execution", include_in_schema=False)
async def test_execution(request: Request, request_data: APIRequestModel):
    target_path = request_data.path

    # 헤더 파싱
    request_headers = {k: str(v) for k, v in (request_data.input.headers or {}).items()} if request_data.input else {}

    # 쿠키 처리
    if request_data.input and getattr(request_data.input, "cookie_params", None):
        if request_data.input.cookie_params is not None:
            cookie_header = "; ".join(f"{k}={v}" for k, v in request_data.input.cookie_params.items())
            request_headers["Cookie"] = cookie_header

    target_path = request_data.path
    if request_data.input and request_data.input.path_params:
        for k, v in request_data.input.path_params.items():
            target_path = target_path.replace(f"{{{k}}}", str(v))
    target_path = urllib.parse.urljoin(str(request.base_url), target_path)

    target_path = urllib.parse.urljoin(str(request.base_url), target_path)
    # 자기 자신의 api 호출
    transport = httpx.ASGITransport(app=request.app)
    async with httpx.AsyncClient(transport=transport) as client:
        try:
            response = await client.request(
                method=request_data.method,
                url=target_path,
                params=request_data.input.query_params if request_data.input else None,
                json=request_data.input.body.value if request_data.input else None,
                headers=request_headers
            )

            # 프록시된 응답에서 헤더를 복사하되, Content-Length 및 Transfer-Encoding은 제외합니다.
            # 이는 JSONResponse가 새 본문에 대해 올바른 길이를 계산하도록 하기 위함입니다.
            proxied_headers = dict(response.headers)
            if "content-length" in proxied_headers:
                del proxied_headers["content-length"]
            if "transfer-encoding" in proxied_headers:
                del proxied_headers["transfer-encoding"]

            # 대상 백엔드에서 에러를 반환했을 경우
            if response.is_client_error or response.is_server_error:
                # 콘텐츠를 JSON으로 파싱 시도, JSON이 아니면 텍스트로 폴백
                try:
                    error_content = response.json()
                except ValueError:
                    error_content = response.text

                return JSONResponse(
                    status_code=response.status_code,
                    content={
                        "success": False,
                        "message": f"An error was returned from the user backend ({response.status_code})",
                        "data": {
                            "code": f"TARGET_BACKEND_ERROR_{response.status_code}",
                            "details": error_content
                        }
                    },
                    headers=proxied_headers, # 수정된 헤더 사용
                    media_type=response.headers.get("content-type", "application/json")
                )

            # 테스트가 성공했을 경우 json 또는 text 응답 반환
            try:
                success_content = response.json()
            except ValueError:
                success_content = response.text

            return JSONResponse(
                status_code=response.status_code,
                content={
                    "success": response.is_success,
                    "message": "test case success." if response.is_success else "test case fail.",
                    "data": success_content,
                },
                headers={k: v for k, v in response.headers.items() if k.lower() not in ["content-length", "transfer-encoding"]},
                media_type=response.headers.get("content-type", "application/json"),
            )

        except httpx.RequestError as e:
            return make_error_response(500, "HTTPX_REQUEST_ERROR", "네트워크/연결 오류", str(e))
        except Exception as e:
            return make_error_response(500, "UNEXPECTED_PROXY_ERROR", "프록시 실행 중 예상치 못한 오류", str(e))



from fastapi.responses import JSONResponse
from typing import Any
def make_error_response(status_code: int, code: str, message: str, details: Any = None):
    return JSONResponse(
        status_code=status_code,
        content={
            "success": False,
            "message": message,
            "data": {"code": code, "details": details},
        }
    )




from fastapi import Request
from fastapi.responses import JSONResponse
from starlette.middleware.base import BaseHTTPMiddleware
import traceback
import logging

logger = logging.getLogger(__name__)

class ExceptionHandlingMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        try:
            response = await call_next(request)
            return response
        except Exception as e:
            logger.error("Unhandled Exception: %s", traceback.format_exc())
            return JSONResponse(
                status_code=500,
                content={
                    "success": False,
                    "message": "Internal Server Error",
                    "error": {
                        "code": "UNEXPECTED_ERROR",
                        "details": str(e)
                    }
                }
            )


from typing import Any, Dict, Generic, TypeVar, Optional
from pydantic import BaseModel
from pydantic.generics import GenericModel

T = TypeVar("T")
class APIResponse(GenericModel, Generic[T]):
    success: bool = True
    message: str
    data: Optional[T] = None

class APIErrorDetail(BaseModel):
    code: str
    details: Optional[Any] = None  # 또는 Dict[str, Any]도 가능

class APIErrorResponse(BaseModel):
    success: bool = False
    message: str
    data: APIErrorDetail




from typing import Optional

class OpenApiService:
    def __init__(self):
        self.current_spec: Optional[dict] = None
        
    def set_current_spec(self, openapi_json: dict):
        self.current_spec = openapi_json
        
    def get_current_spec(self) -> dict:
        if self.current_spec is None:
            return {}
        return self.current_spec
        


import logging

class NoZyloDocsLogFilter(logging.Filter):
    def filter(self, record: logging.LogRecord) -> bool:
        if record.name == "uvicorn.access" and isinstance(record.args, tuple) and len(record.args) >= 3:
            path = record.args[2]
            if isinstance(path, str) and path.startswith("/zylo-docs"):
                return False
        return True

    def setup_logging(self):
        """uvicorn 로거에 필터를 적용합니다."""
        logging.getLogger("uvicorn.access").addFilter(NoZyloDocsLogFilter())



from fastapi import APIRouter, Request,Response, Query
from typing import Optional
import httpx
from pydantic import BaseModel, Field
from enum import Enum
from zylo_docs.services.openapi_service import OpenApiService
from zylo_docs.config import EXTERNAL_API_BASE
from pydantic import BaseModel
import logging
logger = logging.getLogger(__name__)

router = APIRouter()
class DocTypeEnum(str, Enum):
    internal = "internal"
    public = "public"
    partner = "partner"

class ZyloAIRequestBody(BaseModel):
    title: str = Field(..., description="Title of the OpenAPI spec")
    version: str = Field(..., description="Version of the spec")
    doc_type: DocTypeEnum
    
class ZyloAIUserContextRequestBody(BaseModel):
    title: str = Field(..., description="Title of the OpenAPI spec")
    version: str = Field(..., description="Version of the spec")
    doc_type: DocTypeEnum
    spec_id: str = Field(..., description="Spec ID for current spec")
    user_context: Optional[str] = Field(None, description="User context for the spec")
    
class InviteRequestBody(BaseModel):
    emails: list[str] = Field(..., description="List of emails to invite")
class TestCasePatchBody(BaseModel):
    spec_id: str = Field(..., description="Spec ID for the test case")
    path: str = Field(..., description="Operation ID for the test case")
    method: str = Field(..., description="Test case method")

            
@router.get("/download-spec", include_in_schema=False)
async def download_current_spec(request: Request, spec_id: str = Query(..., description="OpenAPI spec ID")):
    service: OpenApiService = request.app.state.openapi_service
    openapi_dict = service.get_current_spec()

    if spec_id == "original":
        return {
            "success": True,
            "message": "Original OpenAPI spec retrieved successfully",
            "data": request.app.openapi()
        }
    else:
        return {
                    "success": True,
                    "message": "Spec retrieved successfully",
                    "data": openapi_dict
                }
    
@router.api_route("/{path:path}", methods=["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS"], include_in_schema=False)
async def proxy(request: Request, path: str):
        async with httpx.AsyncClient() as client:
            proxy_url = f"{EXTERNAL_API_BASE}/{path}"
            body = await request.body()
            headers = dict(request.headers)
            headers.pop("host", None) 
            
            resp = await client.request(
                method=request.method,
                url=proxy_url,
                content=body,
                headers=headers,
                params=request.query_params,
            )
            
        headers_to_frontend = dict(resp.headers)
        # 프론트로 보내는 응답 객체 프론트와 인터페이스를 맞춰야함
        return Response(
            status_code=resp.status_code,
            headers=headers_to_frontend,
            content=resp.content,
            media_type=resp.headers.get("content-type")
        )




import os
from dotenv import load_dotenv

# config.py 파일의 위치를 기준으로 상위 폴더의 .env 파일을 찾습니다.
dotenv_path = os.path.join(os.path.dirname(__file__), "..", ".env")
load_dotenv(dotenv_path=dotenv_path)

EXTERNAL_API_BASE = os.getenv("EXTERNAL_API_BASE", "https://v2-api.zylosystems.com/v1")
